#ifndef CORE_ARRAY_HPP
#define CORE_ARRAY_HPP

#include "core/utils/ArrayString.hpp"

namespace Core {
    template<typename T, size_t N>
    class Array final {
        T data[N];

    public:
        constexpr Array() = default;

        constexpr Array(const T& t) {
            fill(t);
        }

        constexpr void fill(const T& t) {
            for(size_t i = 0; i < N; i++) {
                data[i] = t;
            }
        }

        constexpr T& operator[](size_t index) {
            return data[index];
        }

        constexpr const T& operator[](size_t index) const {
            return data[index];
        }

        constexpr T* begin() {
            return data;
        }

        constexpr T* end() {
            return data + N;
        }

        constexpr const T* begin() const {
            return data;
        }

        constexpr const T* end() const {
            return data + N;
        }

        static constexpr size_t getLength() {
            return N;
        }

        template<typename String>
        check_return Error toString(String& s) const {
            return Core::toString(s, *this);
        }
    };
}

#endif
